@@ -1,60 +1,23 @@
-#ABSTRACT: Calculate N50 from a FASTA or FASTQ file without dependencies
+#ABSTRACT: Lightweight module to calculate N50 statistics from a FASTA or FASTQ file
 
 use 5.016;
 use warnings;
 
 package Proch::N50;
-$Proch::N50::VERSION = '0.03';
-
+$Proch::N50::VERSION = '0.05';
+use JSON::PP;
+use FASTX::Reader;
 use File::Basename;
 use Exporter qw(import);
 our @EXPORT = qw(getStats getN50 jsonStats);
 
 
-my $hasJSON = 0;
-
-$hasJSON = eval {
-    require JSON;
-    JSON->import();
-    1;
-};
-
-sub _n50fromHash {
-    my ( $hash_ref, $total ) = @_;
-    my $tlen = 0;
-    foreach my $s ( sort { $a <=> $b } keys %{$hash_ref} ) {
-        $tlen += $s * ${$hash_ref}{$s};
-
-     # Was '>=' in my original implementation of N50. Now complies with 'seqkit'
-        return $s if ( $tlen > ( $total / 2 ) );
-    }
-
-}
-
-sub getN50 {
-
-    # Invokes the full getStats returning N50 or 0 in case of error;
-    my ($file) = @_;
-    my $stats = getStats($file);
-    if ( $stats->{status} ) {
-        return $stats->{N50};
-    }
-    else {
-        return 0;
-    }
-}
-sub jsonStats {
-  my ($file) = @_;
-  my $stats = getStats($file,  'JSON');
-  if ($stats->{status} and $stats->{json}) {
-    return $stats->{json}
-  } else {
-    return undef;
-  }
-}
 sub getStats {
-
     # Parses a FASTA/FASTQ file and returns stats
+    # Parameters:
+    # * filename (Str)
+    # * Also return JSON string (Bool)
+
     my ( $file, $wantJSON ) = @_;
     my $answer;
     $answer->{status} = 1;
@@ -66,93 +29,106 @@ sub getStats {
         $answer->{message} = "Unable to find <$file>";
     }
 
-    open FILE, '<', "$file" || do {
-        $answer->{status}  = 0;
-        $answer->{message} = "Unable to read <$file>";
-    };
+    
 
+    # Return failed status if file not found or not readable
     if ( $answer->{status} == 0 ) {
         return $answer;
     }
 
-    my @aux = undef;
+    ##my @aux = undef;
+    my $Reader;
+    if ($file ne '-') {
+       $Reader = FASTX::Reader->new({ filename => "$file" });
+    } else {
+       $Reader = FASTX::Reader->new({ filename => '{{STDIN}}' });
+    }
     my %sizes;
     my ( $n, $slen ) = ( 0, 0 );
 
     # Parse FASTA/FASTQ file
-    while ( my ( $name, $seq ) = _readfq( \*FILE, \@aux ) ) {
+    while ( my $seq = $Reader->getRead() ) {
         ++$n;
-        my $size = length($seq);
+        my $size = length($seq->{seq});
         $slen += $size;
         $sizes{$size}++;
     }
-    my $n50 = _n50fromHash( \%sizes, $slen );
+
+    # Invokes core _n50fromHash() routine
+    my ($n50, $min, $max) = _n50fromHash( \%sizes, $slen );
 
     my $basename = basename($file);
 
     $answer->{N50}      = $n50;
+    $answer->{min}      = $min;
+    $answer->{max}      = $max;
     $answer->{seqs}     = $n;
     $answer->{size}     = $slen;
     $answer->{filename} = $basename;
     $answer->{dirname}  = dirname($file);
 
-    if ( $hasJSON and defined $wantJSON ) {
-        my $json           = JSON->new->allow_nonref;
-        my $pretty_printed = $json->pretty->encode($answer);
+    # If JSON is required return JSON
+    if ( defined $wantJSON ) {
+
+        my $json = JSON::PP->new->ascii->pretty->allow_nonref;
+        my $pretty_printed = $json->encode( $answer );
         $answer->{json} = $pretty_printed;
 
     }
     return $answer;
 }
 
-sub _readfq {
-    my ( $fh, $aux ) = @_;
-    @$aux = [ undef, 0 ] if ( !(@$aux) );
-    return if ( $aux->[1] );
-    if ( !defined( $aux->[0] ) ) {
-        while (<$fh>) {
-            chomp;
-            if ( substr( $_, 0, 1 ) eq '>' || substr( $_, 0, 1 ) eq '@' ) {
-                $aux->[0] = $_;
-                last;
-            }
-        }
-        if ( !defined( $aux->[0] ) ) {
-            $aux->[1] = 1;
-            return;
-        }
+sub _n50fromHash {
+    # _n50fromHash(): calculate stats from hash of lengths
+    #
+    # Parameters:
+    # * A hash of  key={contig_length} and value={no_contigs}
+    # * Sum of all contigs sizes
+    my ( $hash_ref, $total ) = @_;
+    my $tlen = 0;
+    my @sorted_keys = sort { $a <=> $b } keys %{$hash_ref};
+
+    # Added in v. 0.039
+    my $max =  $sorted_keys[-1];
+    my $min =  $sorted_keys[0] ;
+
+    foreach my $s ( @sorted_keys ) {
+        $tlen += $s * ${$hash_ref}{$s};
+
+     # Was '>=' in my original implementation of N50. Now complies with 'seqkit'
+        return ($s, $min, $max) if ( $tlen > ( $total / 2 ) );
     }
 
-    my $name = '';
-    if ( defined $_ ) {
-        $name = /^.(\S+)/ ? $1 : '';
-    }
-
-    my $seq = '';
-    my $c;
-    $aux->[0] = undef;
-    while (<$fh>) {
-        chomp;
-        $c = substr( $_, 0, 1 );
-        last if ( $c eq '>' || $c eq '@' || $c eq '+' );
-        $seq .= $_;
-    }
-    $aux->[0] = $_;
-    $aux->[1] = 1 if ( !defined( $aux->[0] ) );
-    return ( $name, $seq ) if ( $c ne '+' );
-    my $qual = '';
-    while (<$fh>) {
-        chomp;
-        $qual .= $_;
-        if ( length($qual) >= length($seq) ) {
-            $aux->[0] = undef;
-            return ( $name, $seq, $qual );
-        }
-    }
-    $aux->[1] = 1;
-    return ( $name, $seq );
 }
 
+sub getN50 {
+
+    # Invokes the full getStats returning N50 or 0 in case of error;
+    my ($file) = @_;
+    my $stats = getStats($file);
+
+    # Verify status and return
+    if ( $stats->{status} ) {
+        return $stats->{N50};
+    } else {
+        return 0;
+    }
+}
+
+sub jsonStats {
+  my ($file) = @_;
+  my $stats = getStats($file,  'JSON');
+
+  # Returns JSON object if it was possible to have ig
+  if ($stats->{status} and $stats->{json}) {
+    return $stats->{json}
+  } else {
+    # Returns 'undef' otherwise
+    return undef;
+  }
+}
+ 
+
 1;
 
 __END__
@@ -163,11 +139,11 @@ __END__
 
 =head1 NAME
 
-Proch::N50 - Calculate N50 from a FASTA or FASTQ file without dependencies
+Proch::N50 - Lightweight module to calculate N50 statistics from a FASTA or FASTQ file
 
 =head1 VERSION
 
-version 0.03
+version 0.05
 
 =head1 SYNOPSIS
 
@@ -183,6 +159,8 @@ version 0.03
   # Will print:
   # %FASTA_stats = (
   #               'N50' => 65,
+  #               'min' => 4,
+  #               'max' => 65,
   #               'dirname' => 'data',
   #               'size' => 130,
   #               'seqs' => 6,
@@ -195,12 +173,11 @@ version 0.03
   print $seq_stats_with_JSON->{json}, "\n";
   # Will print:
   # {
-  #    "seqs" : 6,
   #    "status" : 1,
+  #    "seqs" : 6,
+  #    <...>
   #    "filename" : "small_test.fa",
   #    "N50" : "65",
-  #    "dirname" : "data",
-  #    "size" : 130
   # }
 
 =head1 NAME
@@ -238,6 +215,22 @@ the actual N50
 
 =over 4
 
+=item I<min> (int)
+
+Minimum length observed in FASTA/Q file
+
+=back
+
+=over 4
+
+=item I<max> (int)
+
+Maximum length observed in FASTA/Q file
+
+=back
+
+=over 4
+
 =item I<seqs> (int)
 
 total number of sequences in the files
@@ -271,19 +264,27 @@ name of the directory containing the input file
 =head2 jsonStats(filepath)
 
 Returns the JSON string with basic stats (same as $result->{json} from I<getStats>(File, JSON)).
-Requires JSON installed.
+Requires JSON::PP installed.
+
+=head2 _n50fromHash(hash, totalsize)
+
+This is an internal helper subroutine that perform the actual N50 calculation, hence its addition
+to the documentation.
+Expects the reference to an hash of sizes C<$size{SIZE} = COUNT> and the total sum of sizes obtained
+parsing the sequences file.
+Returns N50, min and max lengths.
 
 =head1 Dependencies
 
 =over 4
 
-=item L<JSON> (optional)
+=item L<JSON::PP> (core)
 
 =back
 
 =over 4
 
-=item L<Term::ANSIColor> (optional; for a demo script)
+=item L<Term::ANSIColor> (optional)  for the n50.pl script
 
 =back
 
